# ID-Broker plugin

## Intro
This package contains the id-broker resource for the ucsschool-apis app. Right now
it allows retrieving information about users, groups and schools.
Internally it sends a request to the kelvin api to fetch said information.

```mermaid
graph TD
client --auth: id-broker-admin--> id-broker-api --auth: kelvin-user--> kelvin
```


## Installation & configuration

The client uses a user named `id-broker-admin` to access the id-broker-api, which in
return needs an 'kelvin-user' to access the kelvin api. These users need to be
created and configured in settings. We also need to tell kelvin about the new
`brokerID` attribute of school users.


### Kelvin
- install the ucsschool-kelvin-rest-api app on your system (e.g. in the umc apps module)
- configure the ucsschool-kelvin-rest-api via the file in `/var/lib/ucs-school-import/configs/kelvin.json` to map in the `brokerID` and allow long usernames:

    ```
    {
        "mapped_udm_properties": ["brokerID"],
        "username": {"max_length": {"default": 50, "student": 50}}
    }
    ```
- set the UCR value for the maximum username length in Kelvins Docker container:

    ```
    univention-app shell ucsschool-kelvin-rest-api ucr set ucsschool/username/max_length=50
    ```
- create an 'kelvin-user', the user that the id-broker-api will use to access the kelvin api. The user can have any username (and password), but needs to be member of the group `ucsschool-kelvin-rest-api-admins`.
- restart the kelvin api with `univention-app restart ucsschool-kelvin-rest-api`.

### ID-Broker Plugin

- make the cleartext user/pass of the 'kelvin-user' known to the provisioning-api in
  its settings file in `/etc/ucsschool/apis/id-broker/provisioning-api.json`:
    ```json
    {
      "host": "FQDN_OF_HOST_WITH_KELVIN_API",
      "username": "kelvin-user-name",
      "password": "kelvin-user-secret",
      "verify_ssl": true
    }

    ```
- create a user with the name `id-broker-admin` with a password of your choice. The
  client interacting with the id-broker-api will use this user to interact with the api.
- append `idbroker` to the list `enabled_plugins` in `/etc/ucsschool/apis/settings.json`
- restart the ucsschool-apis app with `univention-app restart ucsschool-apis`.

### Bettermarks Plugin

- go through the steps to install the id-broker-plugin. Adding `idbroker` to `/etc/ucsschool/apis/settings.json` is only needed if you want to use this plugin.
- add `bettermarks`  to the list `enabled_plugins` in `/etc/ucsschool/apis/settings.json`.
- restart the ucsschool-apis app with `univention-app restart ucsschool-apis`.


### Provisioning Plugin

- append `provisioning` to the list `enabled_plugins` in `/etc/ucsschool/apis/settings.json`
- create a user with the name `provisioning-YOURAUTHORITY` for each authority
- restart the ucsschool-apis app with `univention-app restart ucsschool-apis`.

**Remark** Names of school authorities which are synced may only have alphanumeric values as well as `_`.

## Swagger UI

The REST API can be accessed interactively through the Swagger UI at https://FQDN/ucsschool/apis/docs

## REST API requests

### Authentication

To use an API resource, a token must first be retrieved:

```shell
curl -i -k -X POST https://FQDN/ucsschool/apis/auth/token \
    -H "Content-Type:application/x-www-form-urlencoded" \
    -d "username=id-broker-admin" \
    -d "password=s3cr3t"
```
The response headers will be:
```shell
HTTP/1.1 200 OK
Date: Sun, 06 Jun 2021 08:46:20 GMT
Server: uvicorn
content-length: 228
content-type: application/json
Via: 1.1 <FQDN>
```
The response body will be:
```json
{
    "access_token":"eyJ0eXAiOiJKV1Qi...",
    "token_type":"bearer"
}
```
### GET user

```shell
curl -i -k -X GET http://10.200.3.20/ucsschool/apis/idbroker/user?broker_id=BROKERID \
    -H 'accept: application/json' \
    -H 'Authorization: Bearer eyJ0eXAiOiJKV1Qi...'
```
The response headers will be:
```shell
HTTP/1.1 200 OK
Date: Sun, 06 Jun 2021 08:52:09 GMT
Server: uvicorn
content-length: 557
content-type: application/json
Via: <FQDN>
```
The response body will be:
```json
{
    "birthday": null,
    "disabled": false,
    "dn": "uid=USERNAME,cn=schueler,cn=users,ou=DEMOSCHOOL,dc=examle,dc=com",
    "email": null,
    "firstname": "Givenname",
    "kelvin_password_hashes": null,
    "lastname": "Surname",
    "name": "username",
    "password": null,
    "record_uid": "RecordUid",
    "roles": ["student"],
    "school": "DEMOSCHOOL",
    "school_classes": {"DEMOSCHOOL": ["1d"]},
    "schools": ["DEMOSCHOOL"],
    "source_uid": "SourceUid",
    "ucsschool_roles": ["student:school:DEMOSCHOOL"],
    "udm_properties": {"brokerID": "BROKERID"},
    "url": "https://FQDN/ucsschool/kelvin/v1/users/mimi.nordenh"
}
```
### GET /users/{user_id}/metadata

```shell
curl -i -k -X GET http://10.200.3.20/ucsschool/apis/bettermarks/users/BROKERID/metadata \
  -H 'accept: application/json' \
  -H 'Authorization: Bearer eyJ0eXAiOiJKV1Qi...'
```

The response headers will be:
```shell
HTTP/1.1 200 OK
Date: Fri,11 Jun 2021 03:21:08 GMT
Server: uvicorn
content-length: 557
content-type: application/json
Via: <FQDN>
```

The response body will be:
```json
{
  "user_id": "BROKERID",
  "username": "USERNAME",
  "firstname": "First",
  "lastname": "Name",
  "type": "student",
  "school_id": "DEMOSCHOOL"
}
```

### GET /users/{user_id}/groups

```shell
curl -i -k -X GET http://10.200.3.20/ucsschool/apis/bettermarks/users/BROKERID/groups \
  -H 'accept: application/json' \
  -H 'Authorization: Bearer eyJ0eXAiOiJKV1Qi...'
```

The response headers will be:
```shell
HTTP/1.1 200 OK
Date: Fri,11 Jun 2021 03:21:08 GMT
Server: uvicorn
content-length: 557
content-type: application/json
Via: <FQDN>
```

The response body will be:
```json
{
  "groups": [
    {
      "group_id": "DEMOSCHOOL-1d",
      "name": "DEMOSCHOOL-1d",
      "student_count": "N"
    }
  ]
}
```

### GET /groups/{group_id}/users


```shell
curl -i -k -X GET http://10.200.3.20/ucsschool/apis/bettermarks/groups/GROUP_ID/users \
  -H 'accept: application/json' \
  -H 'Authorization: Bearer eyJ0eXAiOiJKV1Qi...'
```

The response headers will be:
```shell
HTTP/1.1 200 OK
Date: Fri,11 Jun 2021 03:21:08 GMT
Server: uvicorn
content-length: 557
content-type: application/json
Via: <FQDN>
```

The response body will be:
```json
{
  "students": [
    {
      "user_id": "BROKERID",
      "username": "demo_student",
      "firstname": "Demo",
      "lastname": "Student"
    }
  ],
  "teachers": [
    {
      "user_id": "BROKERID",
      "username": "demo_teacher"
    }
  ]
}
```