# Copyright 2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

test_kelvin_and_uas_apis_apps_installed () {
  if  ! univention-app status ucsschool-kelvin-rest-api 1>/dev/null 2>/dev/null  ||  ! univention-app status ucsschool-apis 1>/dev/null 2>/dev/null; then
    echo "ERROR: -------------------------------------------------------------------------"
    echo "ERROR: The apps 'UCS@school Kelvin Rest API' (ucsschool-kelvin-rest-api) and"
    echo "ERROR: 'UCS@school APIs' (ucsschool-apis) have to be installed."
    echo "ERROR: Please install the apps manually:"
    echo "ERROR: root@$(hostname -s):~# univention-app install ucsschool-kelvin-rest-api"
    echo "ERROR: root@$(hostname -s):~# univention-app install ucsschool-apis"
    echo "ERROR: -------------------------------------------------------------------------"
    return 1
  fi
  return 0
}

add_prop_to_kelvin_mapped_udm_properties_user () {
  local prop="${1:-}" path="${2:-}"
  python -c "
import json
conf = json.load(open('$path'))
if '$prop' not in conf:
  conf['user'] = ['$prop']
else:
  conf_user = set(conf['user'])
  conf_user.add('$prop')
  conf['user'] = sorted(conf_user)
json.dump(conf, open('$path', 'w'))
"
}

prop_in_kelvin_mapped_udm_properties_user () {
  local prop="${1:-}" path="${2:-}"
  python -c "
import json, sys
conf = json.load(open('$path'))
sys.exit(int('$prop' not in conf.get('user', [])))
"
}

add_id_prop_to_kelvin_mapped_udm_properties_user () {
  local id_prop="${1:-}" path="${2:-}"
  if prop_in_kelvin_mapped_udm_properties_user "$id_prop" "$path";
  then
    echo "Property '$id_prop' already in Kelvin REST APIs configuration file '$path'."
  else
    echo "Adding '$id_prop' to Kelvin REST APIs configuration file '$path'..."
    add_prop_to_kelvin_mapped_udm_properties_user "$id_prop" "$path" || return 1

    echo "Restarting Kelvin REST API app..."
    univention-app restart ucsschool-kelvin-rest-api
  fi
}

plugin_enabled_in_ucsschool_apis () {
  local plugin="${1:-}" path="${2:-}"
  python -c "
import json, sys
conf = json.load(open('$path'))
sys.exit(int('$plugin' not in conf.get('enabled_plugins', [])))
"
}

enable_plugin_in_ucsschool_apis () {
  local plugin="${1:-}" path="${2:-}"
  python -c "
import json
conf = json.load(open('$path'))
conf_plugins = set(conf.get('enabled_plugins', []))
conf_plugins.add('$plugin')
conf['enabled_plugins'] = sorted(conf_plugins)
json.dump(conf, open('$path', 'w'))
"
}

enable_plugin_in_ucsschool_apis_and_restart_uas_apis_app () {
  local plugin="${1:-}" path="${2:-}"
  if plugin_enabled_in_ucsschool_apis "$plugin" "$path";
  then
    echo "Plugin '$plugin' already active in UCS@school APIs configuration file '$path'."
  else
    echo "Adding '$plugin' to UCS@school APIs configuration file '$path'..."
    enable_plugin_in_ucsschool_apis "$plugin" "$path" || return 1

    echo "Restarting UCS@school APIs app..."
    univention-app restart ucsschool-apis
  fi
}

write_secret () {
  local secret="${1:-}" filename="${2:-}"
  mkdir -p "$(dirname $filename)"
  touch "$filename"
  chmod 0600 "$filename"
  chown root:root "$filename"
  echo "$secret" >"$filename"
}

create_empty_settings_file_if_missing () {
  local path="${1:-}"
  if [ ! -f "$path" ]; then
    echo "Creating empty settings file '$path'..."
    mkdir -p -m 750 -v "$(dirname $path)"
    echo "{}" > "$path"
    chown root:root "$path"
    chmod 640 "$path"
  fi
}

set_max_username_length_for_kelvin_and_restart_kelvin () {
  local num="${1:-}"
  univention-app shell ucsschool-kelvin-rest-api ucr set "ucsschool/username/max_length=$num"
  univention-app shell ucsschool-kelvin-rest-api /etc/init.d/ucsschool-kelvin-rest-api restart
}

create_ext_attr_for_pseudonym () {
  local attr_name="${1:-}" sp_name="${2:-}" udm_module="${3:-}" position="${4:-}"
  shift; shift; shift; shift
  module_word="$(echo "$udm_module" | cut -d '/' -f 2)"

  univention-directory-manager settings/extended_attribute create "$@" --ignore_exists \
      --position "cn=IDBroker,cn=custom attributes,cn=univention,$ldap_base" \
      --set name="${attr_name}${module_word}" \
      --set CLIName="$attr_name" \
      --set module="$udm_module" \
      --set tabName="UCS@school" \
      --set tabPosition="$position" \
      --set shortDescription="ID Broker pseudonym for ${sp_name}." \
      --set longDescription="ID Broker pseudonym for ${sp_name}." \
      --set translationShortDescription="\"de_DE\" \"ID Broker Pseudonym für ${sp_name}.\"" \
      --set translationLongDescription="\"de_DE\" \"ID Broker Pseudonym für ${sp_name}.\"" \
      --set syntax=ascii \
      --set mayChange=1 \
      --set multivalue=0 \
      --set ldapMapping="$attr_name" \
      --set objectClass="idBrokerPseudonymsType" || die
}

add_indices_for_attrs () {
  local last_attr="$(echo $@ | awk '{print $NF}')"

  if [ -x /etc/init.d/slapd ] &&
     [ -x /usr/share/univention-ldap/ldap_setup_index ] &&
     [ -e /var/univention-join/joined ] &&
     ! ucr get ldap/index/eq | tr , '\n' | grep -Fxq "$last_attr" &&
     univention-ldapsearch -LLL -o ldif-wrap=no -b cn=Subschema -s base attributeTypes | grep -Fq "NAME '$last_attr'"
  then
    echo "Updating LDAP indices..."
    for attr_name in $@; do
      index_args="$index_args --add-eq ${attr_name}"
    done
    unset stop_and_start_slapd
    pidof slapd >/dev/null 2>/dev/null && stop_and_start_slapd=yes
    [ -n "$stop_and_start_slapd" ] && service slapd stop
    /usr/share/univention-ldap/ldap_setup_index --verbose $index_args || die
    [ -n "$stop_and_start_slapd" ] && service slapd start
    echo "Finished updating LDAP indices."
  else
    echo "Not updating LDAP indices (LDAP server not running or index exists)."
  fi
}
